@@ -153,12 +153,15 @@ Arg::Arg(int argc, char *argv[])
   d_setchatcolors(std::vector<std::pair<long long int, std::string>>()),
   d_mapxmlcontactnames(std::vector<std::pair<std::string, std::string>>()),
   d_mapxmlcontactnamesfromfile(std::string()),
+  d_mapxmladdresses(std::vector<std::pair<std::string, std::string>>()),
+  d_mapxmladdressesfromfile(std::string()),
   d_xmlautogroupnames(false),
   d_setcountrycode(std::string()),
   d_compactfilenames(false),
   d_generatedummy(std::string()),
   d_targetisdummy(false),
   d_htmlignoremediatypes(std::vector<std::string>()),
+  d_pagemenu(false),
   d_input_required(false)
 {
   // vector to hold arguments
@@ -1873,6 +1876,37 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
       }
       continue;
     }
+    if (option == "--mapxmladdresses")
+    {
+      if (i < argsize - 1)
+      {
+        std::string error;
+        if (!parsePairList(arguments[++i], "=", &d_mapxmladdresses, &error))
+        {
+          std::cerr << "[ Error parsing command line option `" << option << "': " << error << " ]" << std::endl;
+          ok = false;
+        }
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
+    if (option == "--mapxmladdressesfromfile")
+    {
+      if (i < argsize - 1)
+      {
+        d_mapxmladdressesfromfile = std::move(arguments[++i]);
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
     if (option == "--xmlautogroupnames")
     {
       d_xmlautogroupnames = true;
@@ -1949,6 +1983,16 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
       }
       continue;
     }
+    if (option == "--pagemenu")
+    {
+      d_pagemenu = true;
+      continue;
+    }
+    if (option == "--no-pagemenu")
+    {
+      d_pagemenu = false;
+      continue;
+    }
     if (option[0] != '-')
     {
       if (d_positionals >= 2)
