@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2022-2024  Selwin van Dijk
+  Copyright (C) 2022-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -130,6 +130,9 @@ R"*(
    --originalfilenames                   Optional modifier for `--exporthtml'. Use the original filenames
                                          for attached media when available. This option can not be used
                                          together with `--append'.
+   --compactfilenames                    Use (very) short filenames for the generated HTML pages. May be
+                                         of use when running into maximum path length limitations on
+                                         Windows.
    --allhtmlpages                        Optional modifier for `--exporthtml'. Convenience option that
                                          enables all the modifying options for `--exporthtml' listed below.
    --themeswitching                      Optional modifier for `--exporthtml'. Adds a button to the HTML
@@ -145,7 +148,7 @@ R"*(
                                          blocked contacts in the database.
    --includesettings                     Optional modifier for `--exporthtml'. Generate a page showing
                                          settings found in the backup file.
-   --includefullcontactslist             Optional modifier for `--exporthtml'. Generate a page showing all
+   --includefullcontactlist              Optional modifier for `--exporthtml'. Generate a page showing all
                                          contacts present in the backups database. These include hidden
                                          and blocked contacts, but also system contacts who might not use
                                          Signal at all.
@@ -154,6 +157,8 @@ R"*(
    --linkify                             Optional modifier for `--exporthtml'. Attempts to turn URLs in
                                          messages actual clickable links. This option is enabled by deault,
                                          and can be disabled by `--no-linkify'.
+   --chatfolders                         Optional modifier for `--exporthtml'. Exports chat folders. This
+                                         option may interact poorly with the `--limitto[xxx]' options.
 --exporttxt <DIRECTORY>                  Export the messages to plain text file. Attachments are omitted.
                                          This option also supports the `--limittothreads',
                                          `--limittothreadbybame', `--limittodates', and `--migratedb'
