@@ -103,8 +103,10 @@ int main(int argc, char *argv[])
 #endif
   {
     if (!ptdb)
-      ptdb.reset(new SignalPlaintextBackupDatabase(xmlfiles, arg.truncate(), arg.verbose(), arg.mapxmlcontactnames(),
-                                                   arg.mapxmlcontactnamesfromfile(), arg.setcountrycode(), arg.xmlautogroupnames()));
+      ptdb.reset(new SignalPlaintextBackupDatabase(xmlfiles, arg.truncate(), arg.verbose(),
+                                                   arg.mapxmlcontactnames(), arg.mapxmlcontactnamesfromfile(),
+                                                   arg.mapxmladdresses(), arg.mapxmladdressesfromfile(),
+                                                   arg.setcountrycode(), arg.xmlautogroupnames()));
     return ptdb->ok();
   };
 
@@ -187,7 +189,7 @@ int main(int argc, char *argv[])
                               arg.stickerpacks(), arg.migratedb(), arg.overwrite(), arg.append(), arg.light(), arg.themeswitching(),
                               arg.addexportdetails(), arg.includeblockedlist(), arg.includefullcontactlist(), false /*arg.includesettings()*/,
                               arg.includereceipts(), arg.originalfilenames(), arg.linkify(), arg.chatfolders(), arg.compactfilenames(),
-                              arg.htmlignoremediatypes()))
+                              arg.pagemenu(), arg.htmlignoremediatypes()))
         return 1;
 
     if (!arg.exportdesktoptxt().empty())
@@ -221,7 +223,7 @@ int main(int argc, char *argv[])
                             arg.stickerpacks(), arg.migratedb(), arg.overwrite(), arg.append(), arg.light(), arg.themeswitching(),
                             arg.addexportdetails(), arg.includeblockedlist(), arg.includefullcontactlist(), false /*arg.includesettings()*/,
                             arg.includereceipts(), arg.originalfilenames(), arg.linkify(), arg.chatfolders(), arg.compactfilenames(),
-                            arg.htmlignoremediatypes()))
+                            arg.pagemenu(), arg.htmlignoremediatypes()))
       return 1;
   }
 
@@ -522,7 +524,7 @@ int main(int argc, char *argv[])
                         arg.setselfid(), arg.includecalllog(), arg.searchpage(), arg.stickerpacks(), arg.migratedb(), arg.overwrite(),
                         arg.append(), arg.light(), arg.themeswitching(), arg.addexportdetails(), arg.includeblockedlist(),
                         arg.includefullcontactlist(), arg.includesettings(), arg.includereceipts(), arg.originalfilenames(),
-                        arg.linkify(), arg.chatfolders(), arg.compactfilenames(), arg.htmlignoremediatypes()))
+                        arg.linkify(), arg.chatfolders(), arg.compactfilenames(), arg.pagemenu(), arg.htmlignoremediatypes()))
       return 1;
 
   if (!arg.exporttxt().empty())
