@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2022-2024  Selwin van Dijk
+  Copyright (C) 2022-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -23,7 +23,7 @@
 
 bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int unique_id, int numattachments, long long int haspreview,
                                      long long int rowid, SqliteDB const &ddb, std::string const &where, std::string const &databasedir,
-                                     bool isquote, bool issticker)
+                                     bool isquote, bool issticker, bool targetisdummy)
 {
   bool quoted_linkpreview = false;
   bool quoted_sticker = false;
@@ -107,6 +107,8 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
                   //"json_extract(json, '" + jsonpath + ".cdnKey') AS cdn_key,"
                   "json_extract(json, '" + jsonpath + ".localKey') AS localKey,"
                   "IFNULL(json_extract(json, '" + jsonpath + ".version'), 1) AS version,"
+                  "IFNULL(json_extract(json, '" + jsonpath + ".width'), 0) AS width,"
+                  "IFNULL(json_extract(json, '" + jsonpath + ".height'), 0) AS height,"
 
                   // only in sticker
                   "json_extract(json, '" + jsonpath + ".emoji') AS sticker_emoji,"
@@ -115,7 +117,7 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
 
                   // not when sticker
                   "json_extract(json, '" + jsonpath + ".fileName') AS file_name,"
-                  "IFNULL(json_extract(json, '" + jsonpath + ".uploadTimestamp'), 0) AS upload_timestamp,"
+                  "IFNULL(JSONLONG(json_extract(json, '" + jsonpath + ".uploadTimestamp')), 0) AS upload_timestamp,"
                   "IFNULL(json_extract(json, '" + jsonpath + ".flags'), 0) AS flags," // currently, the only flag implemented in Signal is:  VOICE_NOTE = 1
                   "IFNULL(json_extract(json, '" + jsonpath + ".pending'), 0) AS pending,"
                   "IFNULL(json_extract(json, '" + jsonpath + ".cdnNumber'), 0) AS cdn_number"
@@ -190,6 +192,8 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
         bepaald::replaceAll(&url, '\b', R"(\b)");
         bepaald::replaceAll(&url, '\f', R"(\f)");
         bepaald::replaceAll(&url, '\r', R"(\r)");
+        bepaald::replaceAll(&url, '\x0B', R"( )");
+        bepaald::replaceAll(&url, '\v', R"(\v)");
         std::string title = linkpreview_results("title");
         bepaald::replaceAll(&title, '\\', R"(\\)");
         bepaald::replaceAll(&title, '/', R"(\/)");
@@ -200,6 +204,8 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
         bepaald::replaceAll(&title, '\b', R"(\b)");
         bepaald::replaceAll(&title, '\f', R"(\f)");
         bepaald::replaceAll(&title, '\r', R"(\r)");
+        bepaald::replaceAll(&title, '\x0B', R"( )");
+        bepaald::replaceAll(&title, '\v', R"(\v)");
         std::string description = linkpreview_results("description");
         bepaald::replaceAll(&description, '\\', R"(\\)");
         bepaald::replaceAll(&description, '/', R"(\/)");
@@ -210,6 +216,8 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
         bepaald::replaceAll(&description, '\b', R"(\b)");
         bepaald::replaceAll(&description, '\f', R"(\f)");
         bepaald::replaceAll(&description, '\r', R"(\r)");
+        bepaald::replaceAll(&description, '\x0B', R"( )");
+        bepaald::replaceAll(&description, '\v', R"(\v)");
 
         SqliteDB::QueryResults jsonstring;
         ddb.exec("SELECT json_array(json_object('url', json('\"" + url + "\"'), 'title', json('\"" + title + "\"'), 'description', json('\"" + description + "\"'), 'date', 0, 'attachmentId', NULL)) AS link_previews",
@@ -252,50 +260,59 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
     }
 
 
-    // get attachment metadata
-    AttachmentMetadata amd;
-    if (version >= 2) [[likely]]
+    long long int filesize = results_attachment_data.valueAsInt(0, "size", 0);
+    std::string hash;
+
+    if (!targetisdummy || filesize == 0)
     {
-      DesktopAttachmentReader dar(version, fullpath, localkey, size);
-#if __cpp_lib_out_ptr >= 202106L
-      std::unique_ptr<unsigned char[]> att_data;
-      if (dar.getAttachmentData(std::out_ptr(att_data), d_verbose) != 0)
-#else
-      unsigned char *att_data = nullptr; // !! NOTE RAW POINTER
-      if (dar.getAttachmentData(&att_data, d_verbose) != 0)
-#endif
+      // get attachment metadata
+      AttachmentMetadata amd;
+      if (version >= 2) [[likely]]
       {
-        Logger::error("Failed to get attachment data");
+        DesktopAttachmentReader dar(version, fullpath, localkey, size);
+#if __cpp_lib_out_ptr >= 202106L
+        std::unique_ptr<unsigned char[]> att_data;
+        if (dar.getAttachmentData(std::out_ptr(att_data), d_verbose) != 0)
+#else
+        unsigned char *att_data = nullptr; // !! NOTE RAW POINTER
+        if (dar.getAttachmentData(&att_data, d_verbose) != 0)
+#endif
+        {
+          Logger::error("Failed to get attachment data");
+          continue;
+        }
+
+#if __cpp_lib_out_ptr >= 202106L
+        amd = AttachmentMetadata::getAttachmentMetaData(fullpath, att_data.get(), size); // get metadata from heap
+#else
+        amd = AttachmentMetadata::getAttachmentMetaData(fullpath, att_data, size);       // get metadata from heap
+        if (att_data)
+          delete[] att_data;
+#endif
+      }
+      else
+        amd = AttachmentMetadata::getAttachmentMetaData(fullpath);                        // get from file
+
+      if (amd.filename.empty() || (amd.filesize == 0 && results_attachment_data.valueAsInt(0, "size", 0) != 0))
+      {
+        Logger::error("Trying to set attachment data. Skipping.");
+        Logger::error_indent("Pending: ", results_attachment_data.valueAsInt(0, "pending"));
+        //results_attachment_data.prettyPrint();
+        //std::cout << amd.filesize << std::endl;
+
+        //std::cout << "Corresponding message:" << std::endl;
+        //ddb.prettyPrint("SELECT DATETIME(ROUND(messages.sent_at/1000),'unixepoch','localtime'),messages.body,COALESCE(conversations.profileFullName,conversations.name) AS correspondent FROM messages LEFT JOIN conversations ON json_extract(messages.json, '$.conversationId') == conversations.id " + where);
         continue;
       }
 
-#if __cpp_lib_out_ptr >= 202106L
-      amd = AttachmentMetadata::getAttachmentMetaData(fullpath, att_data.get(), size); // get metadata from heap
-#else
-      amd = AttachmentMetadata::getAttachmentMetaData(fullpath, att_data, size);       // get metadata from heap
-      if (att_data)
-        delete[] att_data;
-#endif
-    }
-    else
-      amd = AttachmentMetadata::getAttachmentMetaData(fullpath);                        // get from file
+      if (amd.filesize == 0)
+      {
+        Logger::warning("Skipping 0 byte attachment. Not supported in Signal Android.");
+        continue;
+      }
 
-    if (amd.filename.empty() || (amd.filesize == 0 && results_attachment_data.valueAsInt(0, "size", 0) != 0))
-    {
-      Logger::error("Trying to set attachment data. Skipping.");
-      Logger::error_indent("Pending: ", results_attachment_data.valueAsInt(0, "pending"));
-      //results_attachment_data.prettyPrint();
-      //std::cout << amd.filesize << std::endl;
-
-      //std::cout << "Corresponding message:" << std::endl;
-      //ddb.prettyPrint("SELECT DATETIME(ROUND(messages.sent_at/1000),'unixepoch','localtime'),messages.body,COALESCE(conversations.profileFullName,conversations.name) AS correspondent FROM messages LEFT JOIN conversations ON json_extract(messages.json, '$.conversationId') == conversations.id " + where);
-      continue;
-    }
-
-    if (amd.filesize == 0)
-    {
-      Logger::warning("Skipping 0 byte attachment. Not supported in Signal Android.");
-      continue;
+      filesize = amd.filesize;
+      hash = amd.hash;
     }
 
     //insert into part
@@ -304,15 +321,15 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
                    {{d_part_mid, mms_id},
                     {d_part_ct, results_attachment_data.value(0, "content_type")},
                     {d_part_pending, 0},
-                    {"data_size", amd.filesize},
+                    {"data_size", filesize},
                     {(d_database.tableContainsColumn(d_part_table, "unique_id") ? "unique_id" : ""), unique_id},
                     {"voice_note", results_attachment_data.isNull(0, "flags") ? 0 : (results_attachment_data.valueAsInt(0, "flags", 0) == 1 ? 1 : 0)},
-                    {"width", amd.width == -1 ? 0 : amd.width},
-                    {"height", amd.height == -1 ? 0 : amd.height},
+                    {"width", results_attachment_data.value(0, "width")},
+                    {"height", results_attachment_data.value(0, "height")},
                     {"quote", isquote ? 1 : 0},
-                    {(d_database.tableContainsColumn(d_part_table, "data_hash") ? "data_hash" : ""), amd.hash},
-                    {(d_database.tableContainsColumn(d_part_table, "data_hash_start") ? "data_hash_start" : ""), amd.hash},
-                    {(d_database.tableContainsColumn(d_part_table, "data_hash_end") ? "data_hash_end" : ""), amd.hash},
+                    {(d_database.tableContainsColumn(d_part_table, "data_hash") ? "data_hash" : ""), hash},
+                    {(d_database.tableContainsColumn(d_part_table, "data_hash_start") ? "data_hash_start" : ""), hash},
+                    {(d_database.tableContainsColumn(d_part_table, "data_hash_end") ? "data_hash_end" : ""), hash},
                     {"upload_timestamp", results_attachment_data.value(0, "upload_timestamp")},      // will be 0 on sticker
                     {"cdn_number", results_attachment_data.value(0, "cdn_number")}, // will be 0 on sticker, usually 0 or 2, but I dont know what it means
                     {"file_name", results_attachment_data.value(0, "file_name")}},
@@ -390,6 +407,8 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
       bepaald::replaceAll(&url, '\b', R"(\b)");
       bepaald::replaceAll(&url, '\f', R"(\f)");
       bepaald::replaceAll(&url, '\r', R"(\r)");
+      bepaald::replaceAll(&url, '\x0B', R"( )");
+      bepaald::replaceAll(&url, '\v', R"(\v)");
       std::string title = linkpreview_results("title");
       bepaald::replaceAll(&title, '\\', R"(\\)");
       bepaald::replaceAll(&title, '/', R"(\/)");
@@ -400,6 +419,8 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
       bepaald::replaceAll(&title, '\b', R"(\b)");
       bepaald::replaceAll(&title, '\f', R"(\f)");
       bepaald::replaceAll(&title, '\r', R"(\r)");
+      bepaald::replaceAll(&title, '\x0B', R"( )");
+      bepaald::replaceAll(&title, '\v', R"(\v)");
       std::string description = linkpreview_results("description");
       bepaald::replaceAll(&description, '\\', R"(\\)");
       bepaald::replaceAll(&description, '/', R"(\/)");
@@ -410,6 +431,8 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
       bepaald::replaceAll(&description, '\b', R"(\b)");
       bepaald::replaceAll(&description, '\f', R"(\f)");
       bepaald::replaceAll(&description, '\r', R"(\r)");
+      bepaald::replaceAll(&description, '\x0B', R"( )");
+      bepaald::replaceAll(&description, '\v', R"(\v)");
 
       SqliteDB::QueryResults jsonstring;
       ddb.exec("SELECT json_array(json_object("
@@ -447,7 +470,7 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
                             std::vector<std::string>{"ROWID:uint64:" + bepaald::toString(new_part_id),
                                                      (d_database.tableContainsColumn(d_part_table, "unique_id") ?
                                                       "ATTACHMENTID:uint64:" + bepaald::toString(unique_id) : ""),
-                                                     "LENGTH:uint32:" + bepaald::toString(amd.filesize)}))
+                                                     "LENGTH:uint32:" + bepaald::toString(filesize)}))
     {
       new_attachment_frame->setReader(new DesktopAttachmentReader(version, fullpath, localkey, size));
       d_attachments.emplace(std::make_pair(new_part_id,
@@ -459,7 +482,7 @@ bool SignalBackup::dtInsertAttachments(long long int mms_id, long long int uniqu
       Logger::error("Failed to create AttachmentFrame for data");
       Logger::error_indent("       rowid       : ", new_part_id);
       Logger::error_indent("       attachmentid: ", unique_id);
-      Logger::error_indent("       length      : ", amd.filesize);
+      Logger::error_indent("       length      : ", filesize);
       Logger::error_indent("       path        : ", databasedir, "/attachments.noindex/",
                            results_attachment_data.valueAsString(0, "path"));
 
