@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2023-2024  Selwin van Dijk
+  Copyright (C) 2023-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -24,32 +24,13 @@ bool SignalBackup::exportTxt(std::string const &directory, std::vector<long long
                              std::vector<std::string> const &daterangelist, std::string const &selfphone [[maybe_unused]],
                              bool migrate, bool overwrite)
 {
+  Logger::message("Starting plaintext export to '", directory, "'");
+
+  // v170 and above should work. Anything below will first migrate (I believe anything down to ~23 should more or less work)
   bool databasemigrated = false;
   MemSqliteDB backup_database;
-
-  // >= 168 will work already? (not sure if 168 and 169 were ever in production, I don't have them at least)
-  if (d_databaseversion == 167)
+  if (d_databaseversion < 170 || migrate)
   {
-    SqliteDB::copyDb(d_database, backup_database);
-    if (!migrateDatabase(167, 170))
-    {
-      Logger::error("Failed to migrate currently unsupported database version (", d_databaseversion, ")."
-                    " Please upgrade your database");
-      SqliteDB::copyDb(backup_database, d_database);
-      return false;
-    }
-    else
-      databasemigrated = true;
-  }
-  else if (d_databaseversion < 167)
-  {
-    if (!migrate)
-    {
-      Logger::error("Currently unsupported database version (", d_databaseversion, ").");
-      Logger::error_indent("Please upgrade your database or append the `--migratedb' option to attempt to");
-      Logger::error_indent("migrate this database to a supported version.");
-      return false;
-    }
     SqliteDB::copyDb(d_database, backup_database);
     if (!migrateDatabase(d_databaseversion, 170)) // migrate == TRUE, but migration fails
     {
@@ -58,10 +39,44 @@ bool SignalBackup::exportTxt(std::string const &directory, std::vector<long long
       SqliteDB::copyDb(backup_database, d_database);
       return false;
     }
-    else
-      databasemigrated = true;
+    databasemigrated = true;
   }
 
+  // // >= 168 will work already? (not sure if 168 and 169 were ever in production, I don't have them at least)
+  // if (d_databaseversion == 167)
+  // {
+  //   SqliteDB::copyDb(d_database, backup_database);
+  //   if (!migrateDatabase(167, 170))
+  //   {
+  //     Logger::error("Failed to migrate currently unsupported database version (", d_databaseversion, ")."
+  //                   " Please upgrade your database");
+  //     SqliteDB::copyDb(backup_database, d_database);
+  //     return false;
+  //   }
+  //   else
+  //     databasemigrated = true;
+  // }
+  // else if (d_databaseversion < 167)
+  // {
+  //   if (!migrate)
+  //   {
+  //     Logger::error("Currently unsupported database version (", d_databaseversion, ").");
+  //     Logger::error_indent("Please upgrade your database or append the `--migratedb' option to attempt to");
+  //     Logger::error_indent("migrate this database to a supported version.");
+  //     return false;
+  //   }
+  //   SqliteDB::copyDb(d_database, backup_database);
+  //   if (!migrateDatabase(d_databaseversion, 170)) // migrate == TRUE, but migration fails
+  //   {
+  //     Logger::error("Failed to migrate currently unsupported database version (", d_databaseversion, ")."
+  //                   " Please upgrade your database");
+  //     SqliteDB::copyDb(backup_database, d_database);
+  //     return false;
+  //   }
+  //   else
+  //     databasemigrated = true;
+  // }
+
   // check if dir exists, create if not
   if (!prepareOutputDirectory(directory, overwrite))
   {
@@ -70,63 +85,6 @@ bool SignalBackup::exportTxt(std::string const &directory, std::vector<long long
     return false;
   }
 
-  // if (!bepaald::fileOrDirExists(directory))
-  // {
-  //   // try to create
-  //   if (!bepaald::createDir(directory))
-  //   {
-  //     std::cout << bepaald::bold_on << "Error" << bepaald::bold_off
-  //               << ": Failed to create directory `" << directory << "'"
-  //               << " (errno: " << std::strerror(errno) << ")" << std::endl; // note: errno is not required to be set by std
-  //     // temporary !!
-  //     {
-  //       std::error_code ec;
-  //       std::filesystem::space_info const si = std::filesystem::space(directory, ec);
-  //       if (!ec)
-  //       {
-  //         std::cout << "Available  : " << static_cast<std::intmax_t>(si.available) << std::endl;
-  //         std::cout << "Backup size: " << d_fd->total() << std::endl;
-  //       }
-  //     }
-  //     if (databasemigrated)
-  //       SqliteDB::copyDb(backup_database, d_database);
-  //     return false;
-  //   }
-  // }
-
-  // // directory exists, but
-  // // is it a dir?
-  // if (!bepaald::isDir(directory))
-  // {
-  //   std::cout << bepaald::bold_on << "Error" << bepaald::bold_off
-  //             << ": `" << directory << "' is not a directory." << std::endl;
-  //   if (databasemigrated)
-  //     SqliteDB::copyDb(backup_database, d_database);
-  //   return false;
-  // }
-
-  // // and is it empty?
-  // if (!bepaald::isEmpty(directory))
-  // {
-  //   if (!overwrite)
-  //   {
-  //     std::cout << bepaald::bold_on << "Error" << bepaald::bold_off
-  //               << ": Directory '" << directory << "' is not empty. Use --overwrite to clear directory before export." << std::endl;
-  //     if (databasemigrated)
-  //       SqliteDB::copyDb(backup_database, d_database);
-  //     return false;
-  //   }
-  //   std::cout << "Clearing contents of directory '" << directory << "'..." << std::endl;
-  //   if (!bepaald::clearDirectory(directory))
-  //   {
-  //     std::cout << bepaald::bold_on << "Error" << bepaald::bold_off
-  //               << ": Failed to empty directory '" << directory << "'" << std::endl;
-  //     if (databasemigrated)
-  //       SqliteDB::copyDb(backup_database, d_database);
-  //     return false;
-  //   }
-  // }
-
   // check and warn about selfid & note-to-self thread
   d_selfid = selfphone.empty() ? scanSelf() : d_database.getSingleResultAs<long long int>("SELECT _id FROM recipient WHERE " + d_recipient_e164 + " = ?", selfphone, -1);
   if (d_selfid == -1)
@@ -179,7 +137,6 @@ bool SignalBackup::exportTxt(std::string const &directory, std::vector<long long
   // handle each thread
   for (int t : threads)
   {
-
     Logger::message("Dealing with thread ", t);
 
     //bool is_note_to_self = false;//(t == note_to_self_thread_id);
@@ -204,7 +161,8 @@ bool SignalBackup::exportTxt(std::string const &directory, std::vector<long long
     // now get all messages
     SqliteDB::QueryResults messages;
     if (!d_database.exec("SELECT "s
-                         "_id, " + d_mms_recipient_id + ", body, "
+                         "_id, " + d_mms_recipient_id + ", "
+                         + (d_database.tableContainsColumn(d_mms_table, "to_recipient_id") ? "to_recipient_id" : "-1") +  " AS to_recipient_id, body, "
                          "date_received, " + d_mms_type + ", "
                          "attcount, reactioncount, mentioncount, "
                          "IFNULL(remote_deleted, 0) AS remote_deleted, "
@@ -258,7 +216,7 @@ bool SignalBackup::exportTxt(std::string const &directory, std::vector<long long
     std::ofstream txtoutput(directory + "/" + filename, std::ios_base::binary);
     if (!txtoutput.is_open())
     {
-      Logger::error("Failed to open '", directory, "/", filename, " for writing.");
+      Logger::error("Failed to open '", directory, "/", filename, "' for writing.");
       if (databasemigrated)
         SqliteDB::copyDb(backup_database, d_database);
       return false;
@@ -310,17 +268,23 @@ bool SignalBackup::exportTxt(std::string const &directory, std::vector<long long
 
       if (Types::isStatusMessage(type) || Types::isCallType(type))
       {
+        // see note in exporthtml
+        long long int target_rid = msg_recipient_id;
+        if ((Types::isIdentityVerified(type) || Types::isIdentityDefault(type)) &&
+            messages.valueAsInt(i, "to_recipient_id") != -1) [[unlikely]]
+          target_rid = messages.valueAsInt(i, "to_recipient_id");
+
         std::string statusmsg;
         if (!body.empty() ||
             !(d_database.tableContainsColumn(d_mms_table, "message_extras") &&
               messages.valueHasType<std::pair<std::shared_ptr<unsigned char []>, size_t>>(i, "message_extras")))
           statusmsg = decodeStatusMessage(body, messages.getValueAs<long long int>(i, "expires_in"), type,
-                                          getRecipientInfoFromMap(&recipient_info, msg_recipient_id).display_name);
+                                          getRecipientInfoFromMap(&recipient_info, target_rid).display_name);
         else if (d_database.tableContainsColumn(d_mms_table, "message_extras") &&
                  messages.valueHasType<std::pair<std::shared_ptr<unsigned char []>, size_t>>(i, "message_extras"))
           statusmsg = decodeStatusMessage(messages.getValueAs<std::pair<std::shared_ptr<unsigned char []>, size_t>>(i, "message_extras"),
                                           messages.getValueAs<long long int>(i, "expires_in"), type,
-                                          getRecipientInfoFromMap(&recipient_info, msg_recipient_id).display_name);
+                                          getRecipientInfoFromMap(&recipient_info, target_rid).display_name);
 
         txtoutput << "[" << readable_date << "] " << "***" << " " << statusmsg <<  '\n';
       }
