@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2019-2024  Selwin van Dijk
+  Copyright (C) 2019-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -51,6 +51,13 @@ void SignalBackup::handleSms(SqliteDB::QueryResults const &results, std::ofstrea
   {
     realtype = results.getValueAs<long long int>(i, "type");
 
+    // skip status messages for now...
+    if (Types::isStatusMessage(realtype))
+    {
+      //std::cout << "Skipping status message " << realtype << std::endl;
+      return;
+    }
+
     // skip 'This group was updated to a New Group'
     if (realtype == Types::GV1_MIGRATION_TYPE)
       return;
@@ -203,6 +210,13 @@ void SignalBackup::handleMms(SqliteDB::QueryResults const &results, std::ofstrea
   {
     realtype = results.getValueAs<long long int>(i, d_mms_type);
 
+    // skip status messages for now...
+    if (Types::isStatusMessage(realtype))
+    {
+      //std::cout << "Skipping status message " << realtype << std::endl;
+      return;
+    }
+
     switch (realtype & Types::BASE_TYPE_MASK)
     {
       case 1:
@@ -430,7 +444,26 @@ void SignalBackup::handleMms(SqliteDB::QueryResults const &results, std::ofstrea
   long long int seen = 0;
   long long int locked = 0;
 
-  long long int text_only = 1;
+  long long int text_only = 0;
+
+  // attachment data
+  long long int mid = getIntOr(results, i, "_id", -1);
+  SqliteDB::QueryResults part_results;
+  if (mid >= 0)
+    d_database.exec("SELECT _id," +
+                    (d_database.tableContainsColumn(d_part_table, "unique_id") ? "unique_id"s : "-1 AS unique_id"s) + ", " +
+                    (d_database.tableContainsColumn(d_part_table, "seq") ? "seq"s : "0 AS seq"s) + ", " +   // seq was removed in dbv215
+                    d_part_ct + ", file_name, " +
+                    d_part_cd + ", " +
+                    //"chset, " +   // chset was removed in dbv215 (always null in earlier dbs)
+                    //"fn, " +      // idem
+                    //"cid, " +     // idem
+                    //"ctt_s, " +   // idem
+                    //"ctt_t "      // idem
+                    d_part_cl +
+                    " FROM " + d_part_table + " WHERE " + d_part_table + "." + d_part_mid + " = ?", mid, &part_results);
+  if (part_results.rows() == 0)
+    text_only = 1;
 
   outputfile << "  <mms "
              << "msg_box=\"" << msg_box << "\" "
@@ -466,12 +499,11 @@ void SignalBackup::handleMms(SqliteDB::QueryResults const &results, std::ofstrea
              << "retr_txt_cs=\"" << retr_txt_cs << "\" "
              << "resp_txt=\"" << resp_txt << "\" "
              << "resp_st=\"" << resp_st << "\" "
-             << "text_only=\"" << text_only << "\">" << '\n';
+             << "text_only=\"" << text_only << "\">"
+             << '\n';
   // << "=\"" <<  << "\" "
 
   /* PART */
-
-  long long int mid = getIntOr(results, i, "_id", -1);
   /* text - The content of the message. */
   long long int expiration = getIntOr(results, i, "expires_in", -1);
   std::string text;
@@ -537,22 +569,7 @@ void SignalBackup::handleMms(SqliteDB::QueryResults const &results, std::ofstrea
                << "></part>" << '\n';
   }
 
-
-  SqliteDB::QueryResults part_results;
-  if (mid >= 0)
-    d_database.exec("SELECT _id," +
-                    (d_database.tableContainsColumn(d_part_table, "unique_id") ? "unique_id"s : "-1 AS unique_id"s) + ", " +
-                    (d_database.tableContainsColumn(d_part_table, "seq") ? "seq"s : "0 AS seq"s) + ", " +   // seq was removed in dbv215
-                    d_part_ct + ", file_name, " +
-                    d_part_cd + ", " +
-                    //"chset, " +   // chset was removed in dbv215 (always null in earlier dbs)
-                    //"fn, " +      // idem
-                    //"cid, " +     // idem
-                    //"ctt_s, " +   // idem
-                    //"ctt_t "      // idem
-                    d_part_cl +
-                    " FROM " + d_part_table + " WHERE " + d_part_table + "." + d_part_mid + " = ?", mid, &part_results);
-
+  // attachments...
   for (unsigned int j = 0; j < part_results.rows(); ++j)
   {
     long long int seq = getIntOr(part_results, j, "seq", 0);
@@ -585,24 +602,23 @@ void SignalBackup::handleMms(SqliteDB::QueryResults const &results, std::ofstrea
     //                         <xs:attribute name="text" type="xs:string" use="required" />
     //                         <xs:attribute name="data" type="xs:string" use="optional" />
 
-    outputfile << "      <part "
-               << "seq=\"" << seq << "\" "
-               << "ct=\"" << ct << "\" "
-               << "name=\"" << name << "\" "
-               << "chset=\"" << chset << "\" "
-               << "cd=\"" << cd << "\" "
-               << "fn=\"" << fn << "\" "
-               << "cid=\"" << cid << "\" "
-               << "cl=\"" << cl << "\" "
-               << "ctt_s=\"" << ctt_s << "\" "
-               << "ctt_t=\"" << ctt_t << "\" "
-               << "text=\"" << "null" << "\"";
-    // getAttachmentData
     long long int rowid = getIntOr(part_results, j, "_id", -1);
     long long int uniqueid = getIntOr(part_results, j, "unique_id", -1);
     auto attachment = d_attachments.find({rowid, uniqueid});
     if (attachment != d_attachments.end())
     {
+      outputfile << "      <part "
+                 << "seq=\"" << seq << "\" "
+                 << "ct=\"" << ct << "\" "
+                 << "name=\"" << name << "\" "
+                 << "chset=\"" << chset << "\" "
+                 << "cd=\"" << cd << "\" "
+                 << "fn=\"" << fn << "\" "
+                 << "cid=\"" << cid << "\" "
+                 << "cl=\"" << cl << "\" "
+                 << "ctt_s=\"" << ctt_s << "\" "
+                 << "ctt_t=\"" << ctt_t << "\" "
+                 << "text=\"" << "null" << "\"";
       // add this for testing, or your xml file will be huge
       outputfile << " data=\"" << Base64::bytesToBase64String(attachment->second->attachmentData(), attachment->second->attachmentSize())/*.substr(0, 50)*/ << "\" ";
       if (!keepattachmentdatainmemory)
@@ -631,7 +647,7 @@ void SignalBackup::handleMms(SqliteDB::QueryResults const &results, std::ofstrea
       if (msg_box == 1) // incoming message
       {
         SqliteDB::QueryResults r2;
-        if (d_database.exec("SELECT " + d_recipient_e164 + " FROM recipient WHERE _id = ?", results.valueAsString(i, d_mms_recipient_id), &r2) && // should be ok to use d_mms_recipient_id, since msb_box = incoming
+        if (d_database.exec("SELECT " + d_recipient_e164 + " FROM recipient WHERE _id = ?", results.valueAsString(i, d_mms_recipient_id), &r2) && // should be ok to use d_mms_recipient_id, since msg_box = incoming
             r2.rows() == 1)
           sender = r2.valueAsString(0, d_recipient_e164);
       }
@@ -684,7 +700,7 @@ bool SignalBackup::exportXml(std::string const &filename, bool overwrite, std::s
 
   // output header
   std::ofstream outputfile(filename, std::ios_base::binary);
-  outputfile << "<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>" << '\n';
+  outputfile << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>" << '\n';
   outputfile << "<?xml-stylesheet type=\"text/xsl\" href=\"sms.xsl\" ?>" << '\n';
 
   SqliteDB::QueryResults sms_results;
@@ -693,14 +709,18 @@ bool SignalBackup::exportXml(std::string const &filename, bool overwrite, std::s
     if (d_database.tableContainsColumn("sms", "protocol") &&
         d_database.tableContainsColumn("sms", "service_center") &&
         d_database.tableContainsColumn("sms", "subject")) // removed in dbv166
-      d_database.exec("SELECT _id,thread_id,protocol,subject,service_center,read,status,date_sent," + d_sms_date_received + "," + d_sms_recipient_id + ",type,body,expires_in FROM sms WHERE "
-                      + d_sms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) AND "
+      d_database.exec("SELECT _id,thread_id,protocol,subject,service_center,read,status,date_sent," + d_sms_date_received + "," + d_sms_recipient_id + ",type,body,expires_in FROM sms "
+                      "WHERE "
+                      + d_sms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) "
+                      "AND "
                       "(type & ?) == 0 AND ((type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ?)",
                       {Types::GROUP_UPDATE_BIT, Types::BASE_INBOX_TYPE, Types::BASE_OUTBOX_TYPE, Types::BASE_SENDING_TYPE, Types::BASE_SENT_TYPE, Types::BASE_SENT_FAILED_TYPE,
                        Types::BASE_PENDING_SECURE_SMS_FALLBACK, Types::BASE_PENDING_INSECURE_SMS_FALLBACK,  Types::BASE_DRAFT_TYPE}, &sms_results);
     else
-      d_database.exec("SELECT _id,thread_id,read,status,date_sent," + d_sms_date_received + "," + d_sms_recipient_id + ",type,body,expires_in FROM sms WHERE "
-                      + d_sms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) AND "
+      d_database.exec("SELECT _id,thread_id,read,status,date_sent," + d_sms_date_received + "," + d_sms_recipient_id + ",type,body,expires_in FROM sms "
+                      "WHERE "
+                      + d_sms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) "
+                      "AND "
                       "(type & ?) == 0 AND ((type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ? OR (type & 0x1F) == ?)",
                       {Types::GROUP_UPDATE_BIT, Types::BASE_INBOX_TYPE, Types::BASE_OUTBOX_TYPE, Types::BASE_SENDING_TYPE, Types::BASE_SENT_TYPE, Types::BASE_SENT_FAILED_TYPE,
                        Types::BASE_PENDING_SECURE_SMS_FALLBACK, Types::BASE_PENDING_INSECURE_SMS_FALLBACK,  Types::BASE_DRAFT_TYPE}, &sms_results);
@@ -717,21 +737,43 @@ bool SignalBackup::exportXml(std::string const &filename, bool overwrite, std::s
                       (d_database.tableContainsColumn(d_mms_table, "message_extras") ? "message_extras, " : "") +
                       "read, ct_l, m_type, m_size, exp, tr_id, st FROM " + d_mms_table +
                       " WHERE "
-                      + d_mms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) AND " +
+                      + d_mms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) "
+                      "AND " +
                       (d_database.tableContainsColumn(d_mms_table, "to_recipient_id") ? "to_recipient_id IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) AND " : "") +
-                      "(" + d_mms_type + " & ?) == 0 AND "
-                      "(base_type IN (?, ?, ?, ?, ?, ?, ?, ?))",
-                      {Types::GROUP_UPDATE_BIT, Types::BASE_INBOX_TYPE, Types::BASE_OUTBOX_TYPE, Types::BASE_SENDING_TYPE, Types::BASE_SENT_TYPE, Types::BASE_SENT_FAILED_TYPE,
-                       Types::BASE_PENDING_SECURE_SMS_FALLBACK, Types::BASE_PENDING_INSECURE_SMS_FALLBACK,  Types::BASE_DRAFT_TYPE}, &mms_results);
+                      "(" + d_mms_type + " & ?) == 0 AND (" + d_mms_type + " & ?) == 0 AND (" + d_mms_type + " & ?) == 0  AND (" + d_mms_type + " & ?) == 0 AND (" + d_mms_type + " & ?) == 0 AND "
+                      "(base_type IN (?, ?, ?, ?, ?, ?, ?, ?)) AND "
+                      "((" + d_mms_type + " & ?) NOT IN (?, ?, ?)) AND "
+                      "((" + d_mms_type + " & ?) NOT IN (?)) AND "
+                      //"((" + d_mms_type + " & ?) != 0) AND "
+                      "(base_type NOT IN (?, ?)) AND "
+                      "(" + d_mms_type + " NOT IN (?, ?, ?, ?, ?, ?, ?, ?, ?, ?))",
+                      {Types::GROUP_UPDATE_BIT, Types::GROUP_V2_BIT, Types::GROUP_QUIT_BIT, Types::END_SESSION_BIT, Types::EXPIRATION_TIMER_UPDATE_BIT,
+                       Types::BASE_INBOX_TYPE, Types::BASE_OUTBOX_TYPE, Types::BASE_SENDING_TYPE, Types::BASE_SENT_TYPE, Types::BASE_SENT_FAILED_TYPE, Types::BASE_PENDING_SECURE_SMS_FALLBACK, Types::BASE_PENDING_INSECURE_SMS_FALLBACK,  Types::BASE_DRAFT_TYPE,
+                       Types::KEY_EXCHANGE_MASK, Types::KEY_EXCHANGE_IDENTITY_UPDATE_BIT, Types::KEY_EXCHANGE_IDENTITY_VERIFIED_BIT, Types::KEY_EXCHANGE_IDENTITY_DEFAULT_BIT,
+                       Types::SPECIAL_TYPES_MASK, Types::SPECIAL_TYPE_MESSAGE_REQUEST_ACCEPTED,
+                       Types::PROFILE_CHANGE_TYPE, Types::JOINED_TYPE,
+                       Types::GROUP_CALL_TYPE, Types::INCOMING_CALL_TYPE, Types::OUTGOING_CALL_TYPE, Types::MISSED_CALL_TYPE, Types::INCOMING_VIDEO_CALL_TYPE, Types::OUTGOING_VIDEO_CALL_TYPE, Types::MISSED_VIDEO_CALL_TYPE, Types::GV1_MIGRATION_TYPE, Types::CHANGE_NUMBER_TYPE, Types::BOOST_REQUEST_TYPE},
+                      &mms_results);
     else
       d_database.exec("SELECT _id,thread_id,date_received," + d_mms_date_sent + "," + d_mms_recipient_id + "," + d_mms_type + ","
                       "(" + d_mms_type + " & " + bepaald::toString(Types::BASE_TYPE_MASK) + ") AS base_type,body,expires_in,read,m_id,sub,ct_t,ct_l,m_type,m_size,rr,read_status,"
                       "m_cls, sub_cs, ct_cls, v, pri, retr_st, retr_txt, retr_txt_cs, d_tm, d_rpt, exp, resp_txt, tr_id, st, resp_st, rpt_a FROM " + d_mms_table + " WHERE "
-                      + d_mms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) AND "
-                      "(" + d_mms_type + " & ?) == 0 AND "
-                      "(base_type IN (?, ?, ?, ?, ?, ?, ?, ?))",
-                      {Types::GROUP_UPDATE_BIT, Types::BASE_INBOX_TYPE, Types::BASE_OUTBOX_TYPE, Types::BASE_SENDING_TYPE, Types::BASE_SENT_TYPE, Types::BASE_SENT_FAILED_TYPE,
-                       Types::BASE_PENDING_SECURE_SMS_FALLBACK, Types::BASE_PENDING_INSECURE_SMS_FALLBACK,  Types::BASE_DRAFT_TYPE}, &mms_results);
+                      + d_mms_recipient_id + " IN (SELECT _id FROM recipient WHERE " + d_recipient_e164 + " IS NOT NULL OR group_id IS NOT NULL) "
+                      "AND "
+                      "(" + d_mms_type + " & ?) == 0 AND (" + d_mms_type + " & ?) == 0  AND (" + d_mms_type + " & ?) == 0 AND (" + d_mms_type + " & ?) == 0 AND (" + d_mms_type + " & ?) == 0 AND "
+                      "(base_type IN (?, ?, ?, ?, ?, ?, ?, ?)) AND "
+                      "((" + d_mms_type + " & ?) NOT IN (?, ?, ?)) AND "
+                      "((" + d_mms_type + " & ?) NOT IN (?)) AND "
+                      //"((" + d_mms_type + " & ?) != 0) AND "
+                      "(base_type NOT IN (?, ?)) AND "
+                      "(" + d_mms_type + " NOT IN (?, ?, ?, ?, ?, ?, ?, ?, ?, ?))",
+                      {Types::GROUP_UPDATE_BIT, Types::GROUP_V2_BIT, Types::GROUP_QUIT_BIT, Types::END_SESSION_BIT, Types::EXPIRATION_TIMER_UPDATE_BIT,
+                       Types::BASE_INBOX_TYPE, Types::BASE_OUTBOX_TYPE, Types::BASE_SENDING_TYPE, Types::BASE_SENT_TYPE, Types::BASE_SENT_FAILED_TYPE, Types::BASE_PENDING_SECURE_SMS_FALLBACK, Types::BASE_PENDING_INSECURE_SMS_FALLBACK,  Types::BASE_DRAFT_TYPE,
+                       Types::KEY_EXCHANGE_MASK, Types::KEY_EXCHANGE_IDENTITY_UPDATE_BIT, Types::KEY_EXCHANGE_IDENTITY_VERIFIED_BIT, Types::KEY_EXCHANGE_IDENTITY_DEFAULT_BIT,
+                       Types::SPECIAL_TYPES_MASK, Types::SPECIAL_TYPE_MESSAGE_REQUEST_ACCEPTED,
+                       Types::PROFILE_CHANGE_TYPE, Types::JOINED_TYPE,
+                       Types::GROUP_CALL_TYPE, Types::INCOMING_CALL_TYPE, Types::OUTGOING_CALL_TYPE, Types::MISSED_CALL_TYPE, Types::INCOMING_VIDEO_CALL_TYPE, Types::OUTGOING_VIDEO_CALL_TYPE, Types::MISSED_VIDEO_CALL_TYPE, Types::GV1_MIGRATION_TYPE, Types::CHANGE_NUMBER_TYPE, Types::BOOST_REQUEST_TYPE},
+                      &mms_results);
   }
 
   //std::string date;
