@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2019-2024  Selwin van Dijk
+  Copyright (C) 2019-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -47,7 +47,7 @@ bool SignalBackup::importThread(SignalBackup *source, long long int thread)
     source->d_database.exec("SELECT * FROM remapped_recipients", &r);
     if (r.rows())
     {
-      warnOnce("Source database contains 'remapped_recipients'. This case may not yet be handled correctly by this program!");
+      Logger::warnOnce("Source database contains 'remapped_recipients'. This case may not yet be handled correctly by this program!");
       if (d_verbose) [[unlikely]]
       {
         for (unsigned int i = 0; i < r.rows(); ++i)
@@ -396,6 +396,27 @@ bool SignalBackup::importThread(SignalBackup *source, long long int thread)
       Logger::message("  Deleted ", count, " existing key values");
   }
 
+  // delete double backup_media_snapshot (v257)
+  // UNTESTED not sure what this does, but if it is going
+  // to be used for incremental backups (backup v2), and
+  // the  media_id is used to determine if an attachment
+  // was already backed up, this needs to be correct to
+  // prevent data loss..
+  if (d_database.containsTable("backup_media_snapshot") && source->d_database.containsTable("backup_media_snapshot"))
+  {
+    SqliteDB::QueryResults res;
+    d_database.exec("SELECT media_id FROM backup_media_snapshot", &res);
+
+    int count = 0;
+    for (unsigned int i = 0; i < res.rows(); ++i)
+    {
+      source->d_database.exec("DELETE FROM backup_media_snapshot WHERE media_id = ?", res.value(i, "media_id"));
+      count += source->d_database.changed();
+    }
+    if (count)
+      Logger::message("  Deleted ", count, " existing backup_media_snapshot entries");
+  }
+
   // the target will have its own job_spec etc...
   if (source->d_database.containsTable("job_spec"))
     source->d_database.exec("DELETE FROM job_spec");
@@ -640,6 +661,16 @@ table|sender_keys|sender_keys|71|CREATE TABLE sender_keys (_id INTEGER PRIMARY K
   {
     Logger::message("  No existing thread found in target database for this recipient, importing.");
 
+    // unpin thread from source database, to prevent uniqueness issues with target
+    if (source->d_database.tableContainsColumn("thread", source->d_thread_pinned) &&
+        d_database.tableContainsColumn("thread", d_thread_pinned))
+    {
+      // before dbv266, 'unpinned' meant the column was set to '0', after dbv266 it was 'NULL'. Make sure to use target's default here!
+      std::string target_pinned_default = d_database.getSingleResultAs<std::string>("SELECT dflt_value FROM pragma_table_info('thread') WHERE name = '" + d_thread_pinned + "'", std::string());
+      if (!source->d_database.exec("UPDATE thread SET " + source->d_thread_pinned + " = " + target_pinned_default))
+        Logger::warning("Failed to unpin threads in source database.");
+    }
+
     // check identities and recipient prefs for presence of values, they may be there (even
     // though no thread was found (for example via a group chat or deleted thread))
     // get identities from target, drop all rows from source that are already present
@@ -875,7 +906,7 @@ table|sender_keys|sender_keys|71|CREATE TABLE sender_keys (_id INTEGER PRIMARY K
     for (unsigned int i = 0; i < res.rows(); ++i)
     {
       source->d_database.exec("DELETE FROM pending_pni_signature_message "
-                              "WHERE recipient_id = ? AND sent_timestamp = ? AND devide_id = ?",
+                              "WHERE recipient_id = ? AND sent_timestamp = ? AND device_id = ?",
                               {res.value(i, "recipient_id"), res.value(i, "sent_timestamp"), res.value(i, "device_id")});
       count += source->d_database.changed();
     }
@@ -943,6 +974,13 @@ table|sender_keys|sender_keys|71|CREATE TABLE sender_keys (_id INTEGER PRIMARY K
                               {res.value(i, "chat_folder_id"), res.value(i, "thread_id")});
   }
 
+  // delete the default chat_folder
+  if (source->d_database.containsTable("chat_folder_membership") &&
+      d_database.containsTable("chat_folder_membership"))
+  {
+    source->d_database.exec("DELETE FROM chat_folder WHERE NAME IS NULL");
+  }
+
   // // export database
   // std::cout << "Writing database..." << std::endl;
   // SqliteDB database("NEWSTYLE.2.sqlite", false);
