@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2019-2024  Selwin van Dijk
+  Copyright (C) 2019-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -26,6 +26,7 @@
 #include <set>
 #include <vector>
 #include <any>
+#include <iterator>
 #if __cpp_lib_ranges >= 201911L
 #include <ranges>
 #endif
@@ -65,6 +66,7 @@ class SqliteDB
     inline size_t rows() const;
     inline size_t columns() const;
     inline void clear();
+    void printSingleLine(long long int row = -1) const;
     void printLineMode(long long int row = -1) const;
     void prettyPrint(bool truncate, long long int row = -1) const;
     void print(long long int row = -1, bool printheader = true) const;
@@ -103,9 +105,11 @@ class SqliteDB
   sqlite3 *d_db;
   sqlite3_vfs *d_vfs;
   mutable sqlite3_stmt *d_stmt; // cache a (prepared) statement for reuse in subsequent transactions
+  mutable char const *d_error_tail;
   std::string d_name;
   // non-owning pointer!
   std::pair<unsigned char *, uint64_t> *d_data;
+  uint32_t d_databasewriteversion;
   bool d_readonly;
   bool d_ok;
   mutable std::map<std::string, bool> d_tables; // cache results of containsTable/tableContainsColumn
@@ -144,16 +148,19 @@ class SqliteDB
   inline bool printLineMode(std::string const &q) const;
   inline bool printLineMode(std::string const &q, std::any const &param) const;
   inline bool printLineMode(std::string const &q, std::vector<std::any> const &params) const;
+  inline bool printSingleLine(std::string const &q) const;
+  inline bool printSingleLine(std::string const &q, std::any const &param) const;
+  inline bool printSingleLine(std::string const &q, std::vector<std::any> const &params) const;
   static bool copyDb(SqliteDB const &source, SqliteDB const &target);
   inline int changed() const;
   inline long long int lastId() const;
-  inline long long int lastInsertRowid() const;
   inline bool containsTable(std::string const &tablename) const;
   inline bool tableContainsColumn(std::string const &tablename, std::string const &columnname) const;
   template <typename... columnnames>
   inline bool tableContainsColumn(std::string const &tablename, std::string const &columnname, columnnames... list) const;
   inline void clearTableCache() const;
   inline void freeMemory();
+  void checkDatabaseWriteVersion() const;
 
  private:
   inline bool initFromFile();
@@ -176,6 +183,7 @@ class SqliteDB
   template <typename T>
   inline bool isType(std::any const &a) const;
   inline bool schemaVersionChanged() const;
+  void setDatabaseWriteVersion();
 
   inline bool registerCustoms() const;
   static inline void tokencount(sqlite3_context *context, int argc, sqlite3_value **argv);
@@ -193,8 +201,10 @@ inline SqliteDB::SqliteDB(std::string const &name, bool readonly)
   d_db(nullptr),
   d_vfs(nullptr),
   d_stmt(nullptr),
+  d_error_tail(nullptr),
   d_name(name),
   d_data(nullptr),
+  d_databasewriteversion(0),
   d_readonly(readonly),
   d_ok(false),
   d_previous_schema_version{}
@@ -207,7 +217,9 @@ inline SqliteDB::SqliteDB(std::pair<unsigned char *, uint64_t> *data)
   d_db(nullptr),
   d_vfs(MemFileDB::sqlite3_memfilevfs(data)),
   d_stmt(nullptr),
+  d_error_tail(nullptr),
   d_data(data),
+  d_databasewriteversion(0),
   d_readonly(true),
   d_ok(false),
   d_previous_schema_version{}
@@ -234,8 +246,10 @@ inline SqliteDB &SqliteDB::operator=(SqliteDB const &other)
     d_db = nullptr;
     d_vfs = nullptr;
     d_stmt = nullptr;
+    d_error_tail = nullptr;
     d_name = ":memory:";
     d_data = nullptr;
+    d_databasewriteversion = other.d_databasewriteversion;
     d_readonly = other.d_readonly;
     d_ok = initFromFile();
     std::strncpy(d_previous_schema_version, other.d_previous_schema_version, 11);
@@ -261,6 +275,8 @@ inline bool SqliteDB::initFromFile()
   if (!initok)
     return false;
 
+  setDatabaseWriteVersion();
+
   return registerCustoms();
 }
 
@@ -273,6 +289,8 @@ inline bool SqliteDB::initFromMemory()
   if (!initok)
     return false;
 
+  setDatabaseWriteVersion();
+
   return registerCustoms();
 }
 
@@ -333,10 +351,39 @@ inline bool SqliteDB::exec(std::string const &q, std::vector<std::any> const &pa
     sqlite3_finalize(d_stmt);
 
     // create new statement
-    if (sqlite3_prepare_v2(d_db, q.c_str(), -1, &d_stmt, nullptr) != SQLITE_OK) [[unlikely]]
+    if (sqlite3_prepare_v2(d_db, q.c_str(), -1, &d_stmt, &d_error_tail) != SQLITE_OK) [[unlikely]]
     {
       Logger::error("During sqlite3_prepare_v2(): ", sqlite3_errmsg(d_db));
-      Logger::error_indent("-> Query: \"", q, "\"");
+      //// old way: just print the error
+      //Logger::error_indent("\"", q, "\"");
+
+      //// newer way: mark the point _around_ the error posistion
+      // long long int error_pos = std::distance(q.c_str(), d_error_tail);
+      // long long int error_start = std::max(0ll, error_pos - 2);
+      // long long int error_end = std::min(error_pos + 2, static_cast<long long int>(q.size()));
+      // Logger::error_indent("-> Query: \"",
+      //                      q.substr(0, error_start),
+      //                      Logger::Control::BOLD,
+      //                      q.substr(error_start, error_end - error_start),
+      //                      Logger::Control::NORMAL,
+      //                      q.substr(error_end),
+      //                      "\"");
+
+      // attempt to mark the token that sqlite choked on
+      long long int error_pos = std::distance(q.c_str(), d_error_tail);
+      long long int error_start = error_pos; // find the token where the error starts...
+      while (error_start > 0 &&
+             ((q[error_start - 1] >= 'a' && q[error_start - 1] <= 'z') ||
+              (q[error_start - 1] >= 'A' && q[error_start - 1] <= 'Z') ||
+              (q[error_start - 1] >= '0' && q[error_start - 1] <= '9')))
+        --error_start;
+      Logger::error_indent("-> Query: \"",
+                           q.substr(0, error_start),
+                           Logger::Control::BOLD,
+                           q.substr(error_start, error_pos - error_start),
+                           Logger::Control::NORMAL,
+                           q.substr(error_pos));
+
       return false;
     }
   }
@@ -650,6 +697,24 @@ inline bool SqliteDB::printLineMode(std::string const &q, std::vector<std::any>
   return ret;
 }
 
+inline bool SqliteDB::printSingleLine(std::string const &q) const
+{
+  return printSingleLine(q, std::vector<std::any>());
+}
+
+inline bool SqliteDB::printSingleLine(std::string const &q, std::any const &param) const
+{
+  return printSingleLine(q, std::vector<std::any>{param});
+}
+
+inline bool SqliteDB::printSingleLine(std::string const &q, std::vector<std::any> const &params) const
+{
+  QueryResults results;
+  bool ret = exec(q, params, &results);
+  results.printSingleLine();
+  return ret;
+}
+
 template <typename T>
 inline bool SqliteDB::isType(std::any const &a) const
 {
@@ -750,48 +815,44 @@ inline long long int SqliteDB::lastId() const
   return sqlite3_last_insert_rowid(d_db);
 }
 
-inline long long int SqliteDB::lastInsertRowid() const
-{
-  return sqlite3_last_insert_rowid(d_db);
-}
-
 inline bool SqliteDB::containsTable(std::string const &tablename) const
 {
-  if (bepaald::contains(d_tables, tablename))
-    return d_tables[tablename];
+  if (auto it = d_tables.find(tablename); it != d_tables.end())
+    return it->second;
 
   QueryResults tmp;
   if (exec("SELECT DISTINCT tbl_name FROM sqlite_master WHERE type = 'table' AND tbl_name = '" + tablename + "'", &tmp) &&
       tmp.rows() > 0)
   {
-    d_tables[tablename] = true;
+    d_tables.emplace(tablename, true);
     return true;
   }
 
-  d_tables[tablename] = false;
+  d_tables.emplace(tablename, false);
   return false;
 }
 
 inline bool SqliteDB::tableContainsColumn(std::string const &tablename, std::string const &columnname) const
 {
-  if (bepaald::contains(d_columns, tablename) &&
-      bepaald::contains(d_columns[tablename], columnname))
-    return d_columns[tablename][columnname];
+  auto it1 = d_columns.find(tablename);
+  if (it1 != d_columns.end())
+    if (auto it2 = it1->second.find(columnname); it2 != it1->second.end())
+      return it2->second;
 
   QueryResults tmp;
   if (exec("SELECT 1 FROM PRAGMA_TABLE_XINFO('" + tablename + "') WHERE name == '" + columnname + "'", &tmp) &&
       tmp.rows() > 0)
   {
-    if (bepaald::contains(d_columns, tablename))
-      d_columns[tablename][columnname] = true;
+    if (it1 != d_columns.end())
+      it1->second.emplace(columnname, true);
     else
-      d_columns[tablename].emplace(std::make_pair(columnname, true));
+      d_columns[tablename].emplace(columnname, true);
     return true;
   }
-  if (bepaald::contains(d_columns, tablename))
-    d_columns[tablename][columnname] = false;
+  if (it1 != d_columns.end())
+    it1->second.emplace(columnname, false);
   else
-    d_columns[tablename].emplace(std::make_pair(columnname, false));
+    d_columns[tablename].emplace(columnname, false);
   return false;
 }
 
