@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2024  Selwin van Dijk
+  Copyright (C) 2024-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -19,6 +19,7 @@
 
 #include "xmldocument.h"
 
+#include <cstring>
 #include <fstream>
 #include <memory>
 
@@ -33,23 +34,33 @@ XmlDocument::XmlDocument(std::string const &filename)
   d_currentnode(&d_rootnode),
   d_ok(false)
 {
-  std::ifstream file(filename);
+  std::ifstream file(filename, std::ios_base::in | std::ios_base::binary);
+  if (!file.is_open()) [[unlikely]]
+  {
+    Logger::error("Failed to open XML file for reading: '", filename, "'");
+    return;
+  }
 
-  unsigned int buffer_size = 4096;
-  std::unique_ptr<char []> buffer(new char[buffer_size]);
-  //unsigned int filepos; // save file position of (very) large values
+  unsigned int constexpr buffer_size = 16 * 1024;
+  //std::unique_ptr<char[]> buffer(new char[buffer_size]);
+  char buffer[buffer_size];
+
+  unsigned long long int filepos; // save file position of (very) large values
   unsigned int available;
   State state = INITIAL;
   State state_before_comment = INITIAL;
   std::string attribute_name_tmp;
   std::string attribute_value_tmp;
+  long long int attribute_pos = -1;
+  long long int attribute_size = 0;
   std::string closing_tag_tmp;
   bool has_root_element = false;
   while (true)
   {
-    //filepos = file.tellg();
-    file.read(buffer.get(), buffer_size);
+    filepos = file.tellg();
+    file.read(buffer, buffer_size);
     available = file.gcount();
+
     for (unsigned int i = 0; i < available; ++i)
     {
       switch (state) // order determined for SignalPlaintextBackup.xml
@@ -57,14 +68,105 @@ XmlDocument::XmlDocument(std::string const &filename)
 
         case ATTRIBUTE_VALUE_DOUBLE:       //                  v
         {                                  //  <tag attribute="...
+          if (attribute_pos == -1) [[unlikely]]
+          {
+            attribute_pos = filepos + i;
+            attribute_size = 0;
+            // if (attribute_name_tmp == "data")
+            //   std::cout << "attribute value starts at pos: " << attribute_pos << "(" << attribute_size << ")" << std::endl;
+          }
+
+          /**** NEW METHOD ****/
+          // just quickly scan for closing quote,
+          // this is not very strict, as we dont
+          // care about invalid characters in the
+          // attribute value ('<' '>' '&'(except in entity))
+          char *closing_quote = static_cast<char *>(std::memchr(buffer + i, '"', available - i));
+          if (closing_quote != nullptr)
+          {
+            attribute_size = attribute_size + (closing_quote - (buffer + i));
+            attribute_value_tmp.append(buffer + i, attribute_size < Node::s_maxsize ? attribute_size - attribute_value_tmp.size() : 0);
+
+            // if (attribute_name_tmp == "data")
+            // {
+            // std::cout << " - attribute: '" << attribute_name_tmp << "'='" << attribute_value_tmp << "'" << std::endl;
+            // std::cout << "filepos : " << filepos << std::endl;
+            // //std::cout << "i       : " << newbufferpos << std::endl;
+            // std::cout << "att_pos : " << attribute_pos << std::endl;
+            // std::cout << "att_size: " << attribute_size << std::endl;
+            // std::cout << "attribute value ends at pos: " << (filepos + i + (closing_quote - (buffer.get() + i))) << " SIZE: " << attribute_size << std::endl;
+            // }
+
+            Node::StringOrRef attributevalue =
+              {
+                (attribute_size < Node::s_maxsize) ? std::move(attribute_value_tmp) : std::string(),
+                (attribute_size < Node::s_maxsize) ? std::string() : filename,
+                (attribute_size < Node::s_maxsize) ? -1 : attribute_pos,
+                attribute_size
+              };
+            // if (attribute_name_tmp == "data")
+            // {
+            // std::cout << attributevalue.file << std::endl;
+            // std::cout << attributevalue.value << std::endl;
+            // std::cout << attributevalue.size << std::endl;
+            // std::cout << attributevalue.pos << std::endl;
+            // }
+            d_currentnode->d_attributes[attribute_name_tmp] = std::move(attributevalue);
+
+            attribute_value_tmp.clear();
+            attribute_name_tmp.clear();
+            attribute_pos = -1;
+
+            i += (closing_quote - (buffer + i));
+
+            //std::cout << "State: ATTRIBUTE_VALUE_DOUBLE -> ELEMENT_AFTER_TAGNAME" << std::endl;
+            state = ELEMENT_AFTER_TAGNAME;
+          }
+          else
+          {
+            attribute_size += available - i;
+            attribute_value_tmp.append(buffer + i, attribute_value_tmp.size() < Node::s_maxsize ? available - i : 0);
+            i = available;
+          }
+          /**** END NEW METHOD ****/
+
+          /**** OLD METHOD
           switch (buffer[i])
           {
-            case '"':
+            [[unlikely]] case '"':
             {
-              //std::cout << " - attribute: '" << attribute_name_tmp << "'='" << attribute_value_tmp << "'" << std::endl;
-              d_currentnode->d_attributes[attribute_name_tmp] = std::move(attribute_value_tmp);
+              attribute_size = (filepos + i) - attribute_pos;
+              // std::cout << " - attribute: '" << attribute_name_tmp << "'='" << attribute_value_tmp << "'" << std::endl;
+              // std::cout << "filepos : " << filepos << std::endl;
+              // std::cout << "i       : " << i << std::endl;
+              // std::cout << "att_pos : " << attribute_pos << std::endl;
+              // std::cout << "att_size: " << attribute_size << std::endl;
+              // std::cout << "attribute value ends at pos: " << (filepos + i) << " SIZE: " << attribute_size << std::endl;
+
+              // {
+              //   if (attribute_size > 0)
+              //   {
+              //     std::ifstream tmp(filename, std::ios_base::in | std::ios_base::binary);
+              //     tmp.seekg(attribute_pos);
+              //     std::unique_ptr<char[]> v(new char[attribute_size]);
+              //     tmp.read(v.get(), attribute_size);
+              //     std::cout << " *** " << std::string(v.get(), attribute_size) << std::endl;
+              //   }
+              // }
+
+              Node::StringOrRef attributevalue =
+                {
+                  (attribute_size < Node::s_maxsize) ? std::move(attribute_value_tmp) : std::string(),
+                  (attribute_size < Node::s_maxsize) ? std::string() : filename,
+                  (attribute_size < Node::s_maxsize) ? -1 : attribute_pos,
+                  attribute_size
+                };
+
+              d_currentnode->d_attributes[attribute_name_tmp] = std::move(attributevalue);
               attribute_value_tmp.clear();
               attribute_name_tmp.clear();
+              attribute_pos = -1;
+
               //std::cout << "State: ATTRIBUTE_VALUE_DOUBLE -> ELEMENT_AFTER_TAGNAME" << std::endl;
               state = ELEMENT_AFTER_TAGNAME;
               break;
@@ -81,6 +183,8 @@ XmlDocument::XmlDocument(std::string const &filename)
               break;
             }
           }
+          END OLD METHOD ****/
+
           break;
         }
 
@@ -106,12 +210,14 @@ XmlDocument::XmlDocument(std::string const &filename)
             case '\n': // this needs special case, as only a '=' is expected after this...
             case '\t': // that is, if we stay in ATTRIBUTE_NAME, the previous case should be denied.
             case '\r':
+            {
               break;
-              [[unlikely]] default:
-              {
-                Logger::error("Illegal character in attribute name: '", buffer[i], "'");
-                return;
-              }
+            }
+            [[unlikely]] default:
+            {
+              Logger::error("Illegal character in attribute name: '", buffer[i], "'");
+              return;
+            }
           }
           break;
         }
@@ -144,13 +250,14 @@ XmlDocument::XmlDocument(std::string const &filename)
             case '\n':
             case '\t':
             case '\r':
+            {
               break;
-
-              [[unlikely]] default:
-              {
-                Logger::error("Illegal character in ELEMENT_AFTER_TAGNAME? ('", buffer[i], "')");
-                return;
-              }
+            }
+            [[unlikely]] default:
+            {
+              Logger::error("Illegal character in ELEMENT_AFTER_TAGNAME? ('", buffer[i], "')");
+              return;
+            }
           }
           break;
         }
@@ -175,12 +282,14 @@ XmlDocument::XmlDocument(std::string const &filename)
             case '\n':
             case '\t':
             case '\r':
+            {
               break;
-              [[unlikely]] default:
-              {
-                Logger::error("Illegal character waiting for attribute value (must be quotation mark): '", buffer[i], "'");
-                return;
-              }
+            }
+            [[unlikely]] default:
+            {
+              Logger::error("Illegal character waiting for attribute value (must be quotation mark): '", buffer[i], "'");
+              return;
+            }
           }
           break;
         }
@@ -441,30 +550,43 @@ XmlDocument::XmlDocument(std::string const &filename)
 
         case ATTRIBUTE_VALUE_SINGLE:       //                 v
         {                                  //  <tag attribute='...
-          switch (buffer[i])
+          if (attribute_pos == -1) [[unlikely]]
           {
-            case '\'':
-            {
-              //std::cout << " - attribute: '" << attribute_name_tmp << "'='" << attribute_value_tmp << "'" << std::endl;
-              d_currentnode->d_attributes[attribute_name_tmp] = std::move(attribute_value_tmp);
-              attribute_value_tmp.clear();
-              attribute_name_tmp.clear();
-              //std::cout << "State: ATTRIBUTE_VALUE_SINGLE -> ELEMENT_AFTER_TAGNAME" << std::endl;
-              state = ELEMENT_AFTER_TAGNAME;
-              break;
-            }
-            [[unlikely]] case '<': // ampersand is also not allowed as a representing itself, but is used for entities (like &amp;)
-            [[unlikely]] case '>':
-            {
-              Logger::error("Illegal character in attribute value: '", buffer[i], "'");
-              return;
-            }
-            default:
-            {
-              attribute_value_tmp += buffer[i];
-              break;
-            }
+            attribute_pos = filepos + i;
+            attribute_size = 0;
           }
+
+          /**** NEW METHOD ****/
+          // See ATTRIBUTE_VALUE_DOUBLE for notes and OLD METHOD
+          char *closing_quote = static_cast<char *>(std::memchr(buffer + i, '\'', available - i));
+          if (closing_quote != nullptr)
+          {
+            attribute_size = attribute_size + (closing_quote - (buffer + i));
+            attribute_value_tmp.append(buffer + i, attribute_size < Node::s_maxsize ? attribute_size - attribute_value_tmp.size() : 0);
+
+            Node::StringOrRef attributevalue = {
+              (attribute_size < Node::s_maxsize) ? std::move(attribute_value_tmp) : std::string(),
+              (attribute_size < Node::s_maxsize) ? std::string() : filename,
+              (attribute_size < Node::s_maxsize) ? -1 : attribute_pos,
+              attribute_size
+            };
+            d_currentnode->d_attributes[attribute_name_tmp] = std::move(attributevalue);
+
+            attribute_value_tmp.clear();
+            attribute_name_tmp.clear();
+            attribute_pos = -1;
+
+            i += (closing_quote - (buffer + i));
+
+            state = ELEMENT_AFTER_TAGNAME;
+          }
+          else
+          {
+            attribute_size += available - i;
+            attribute_value_tmp.append(buffer + i, attribute_value_tmp.size() < Node::s_maxsize ? available - i : 0);
+            i = available;
+          }
+          /**** END NEW METHOD ****/
           break;
         }
 
